// SuperCollider sample class definition

// MyClass is the name of the class. It starts with a Cap letter

MyClass { // class definition is defined by curly braces
	// ALL variables in a class need to be defined within the scope of the Class, or within any 
	// class inheritance. More on that below
	
	// classvar are variables accessible to ALL instances of a class. They use the keyword 'classvar'
	// '<" lets you 'get' a value from a variable
	// '>' lets you 'set' a value... '<>' lets you do both
	classvar <>someValue; 
	// instance variables are slots of memory that are local to an instance of a class.
	var <whatImWorth; 
	
	// method definitions... methods beginning with a * are class methods
	*new {arg initVal;
		// a '^' indicates a return from a method. 'new' methods should return a new instance
		// of a Class. super.new refers here to Object (the implied SuperClass... everything inherits 
		// from Object). This allocates the memory needed for the class to work, and returns it
		// back to you language environment for you to use
		^super.new.initMyClass(initVal); // create a new Object of class MyClass, and run the 
									// initMyClass instance method defined below
	}
	
	initMyClass {arg initVal;
		// store initVal in the whatImWorth slot!
		whatImWorth = initVal;
	}
	
	stelios {
		"Stelios is a badass".postln;
	}
	
	// notice whatImWorth doesn't have a setter! We can also define one here if the method needs
	// to be more complex then simply setting a value... note the underscore _
	
	whatImWorth_ {arg newVal;
		"I'm setting a new value!".postln;
		whatImWorth = newVal;
	}
	
	// return what I'm worth times someValue
	whatImWorthTimesSomeValue {
		^whatImWorth * someValue;
	}
	
	// *initClass Class Methods are called when the library compiles = you can initialize things here
	// that are classvars
	*initClass {
		someValue = 100
	}
}

MyClass2 : MyClass {

	whatImWorthPlusSomeValue {
		^whatImWorth + someValue;
	}	
	
}

Pretify : UGen {
	*ar {arg input, index, mul = 1, add = 0;
		^this.multiNew('audio', input, index).madd(mul, add)
	}	
}










